unit mckCCtrls;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, KOLCCtrls, mirror;

{$I KOLDEF.INC}

type
  TKOLTrackBar = class( TKOLControl )
  private
    FOptions: TTrackbarOptions;
    FPosition: Integer;
    FRangeMin: Integer;
    FSelStart: Integer;
    FThumbLen: Integer;
    FRangeMax: Integer;
    FLineSize: Integer;
    FPageSize: Integer;
    FSelEnd: Integer;
    FOnScroll: TOnScroll;
    FTickFreq: Integer;
    FNumTicks: Integer;
    procedure SetOptions(const Value: TTrackbarOptions);
    procedure SetPosition(const Value: Integer);
    procedure SetLineSize(const Value: Integer);
    procedure SetPageSize(const Value: Integer);
    procedure SetRangeMax(const Value: Integer);
    procedure SetRangeMin(const Value: Integer);
    procedure SetSelEnd(const Value: Integer);
    procedure SetSelStart(const Value: Integer);
    procedure SetThumbLen(const Value: Integer);
    procedure SetOnScroll(const Value: TOnScroll);
    procedure SetNumTicks(const Value: Integer);
    procedure SetTickFreq(const Value: Integer);
  protected
    function AdditionalUnits: String; override;
    function TabStopByDefault: Boolean; override;
    procedure SetupFirst( SL: TStringList; const AName, AParent, Prefix: String ); override;
    function SetupParams( const AName, AParent: String ): String; override;
    procedure SetupConstruct( SL: TStringList; const AName, AParent, Prefix: String ); override;
  public
    constructor Create( AOwner: TComponent ); override;
  published
    property Options: TTrackbarOptions read FOptions write SetOptions;
    property Position: Integer read FPosition write SetPosition;
    property RangeMin: Integer read FRangeMin write SetRangeMin;
    property RangeMax: Integer read FRangeMax write SetRangeMax;
    property PageSize: Integer read FPageSize write SetPageSize;
    property LineSize: Integer read FLineSize write SetLineSize;
    property ThumbLen: Integer read FThumbLen write SetThumbLen;
    property SelStart: Integer read FSelStart write SetSelStart;
    property SelEnd: Integer read FSelEnd write SetSelEnd;
    property OnScroll: TOnScroll read FOnScroll write SetOnScroll;
    property NumTicks: Integer read FNumTicks write SetNumTicks;
    property TickFreq: Integer read FTickFreq write SetTickFreq;
    property TabStop;
    property TabOrder;
  end;

procedure Register;

(*)
{$R mckCCtrls.dcr}
(*)

implementation

procedure Register;
begin
  RegisterComponents( 'KOL', [ TKOLTrackBar ] );
end;

{ TKOLTrackBar }

function TKOLTrackBar.AdditionalUnits: String;
begin
  Result := ', KOLCCtrls';
end;

constructor TKOLTrackBar.Create(AOwner: TComponent);
begin
  inherited;
  Width := 200; DefaultWidth := Width;
  Height := 40; DefaultHeight := Height; 
end;

procedure TKOLTrackBar.SetLineSize(const Value: Integer);
begin
  FLineSize := Value;
  Change;
end;

procedure TKOLTrackBar.SetNumTicks(const Value: Integer);
begin
  FNumTicks := Value;
  Change;
end;

procedure TKOLTrackBar.SetOnScroll(const Value: TOnScroll);
begin
  FOnScroll := Value;
  Change;
end;

procedure TKOLTrackBar.SetOptions(const Value: TTrackbarOptions);
begin
  FOptions := Value;
  Change;
end;

procedure TKOLTrackBar.SetPageSize(const Value: Integer);
begin
  FPageSize := Value;
  Change;
end;

procedure TKOLTrackBar.SetPosition(const Value: Integer);
begin
  FPosition := Value;
  Change;
end;

procedure TKOLTrackBar.SetRangeMax(const Value: Integer);
begin
  FRangeMax := Value;
  Change;
end;

procedure TKOLTrackBar.SetRangeMin(const Value: Integer);
begin
  FRangeMin := Value;
  Change;
end;

procedure TKOLTrackBar.SetSelEnd(const Value: Integer);
begin
  FSelEnd := Value;
  Change;
end;

procedure TKOLTrackBar.SetSelStart(const Value: Integer);
begin
  FSelStart := Value;
  Change;
end;

procedure TKOLTrackBar.SetThumbLen(const Value: Integer);
begin
  FThumbLen := Value;
  Change;
end;

procedure TKOLTrackBar.SetTickFreq(const Value: Integer);
begin
  FTickFreq := Value;
  Change;
end;

procedure TKOLTrackBar.SetupConstruct(SL: TStringList; const AName,
  AParent, Prefix: String);
var S: String;
begin
  S := GenerateTransparentInits;
  SL.Add( Prefix + AName + ' := PTrackbar( New' + TypeName + '( '
          + SetupParams( AName, AParent ) + ' )' + S + ');' );
end;

procedure TKOLTrackBar.SetupFirst(SL: TStringList; const AName, AParent,
  Prefix: String);
begin
  inherited;
  if RangeMin <> 0 then
    SL.Add( Prefix + AName + '.RangeMin := ' + IntToStr( RangeMin ) + ';' );
  if RangeMax <> 0 then
    SL.Add( Prefix + AName + '.RangeMax := ' + IntToStr( RangeMax ) + ';' );
  if PageSize <> 0 then
    SL.Add( Prefix + AName + '.PageSize := ' + IntToStr( PageSize ) + ';' );
  if LineSize <> 0 then
    SL.Add( Prefix + AName + '.LineSize := ' + IntToStr( LineSize ) + ';' );
  if Position <> 0 then
    SL.Add( Prefix + AName + '.Position := ' + IntToStr( Position ) + ';' );
  if ThumbLen <> 0 then
    SL.Add( Prefix + AName + '.ThumbLen := ' + IntToStr( ThumbLen ) + ';' );
  if SelStart <> 0 then
    SL.Add( Prefix + AName + '.SelStart := ' + IntToStr( SelStart ) + ';' );
  if SelEnd <> 0 then
    SL.Add( Prefix + AName + '.SelEnd := '   + IntToStr( SelEnd )   + ';' );
  if TickFreq <> 0 then
    SL.Add( Prefix + AName + '.TickFreq := ' + IntToStr( TickFreq ) + ';' )
  else
  if NumTicks <> 0 then
    SL.Add( Prefix + AName + '.NumTicks := ' + IntToStr( NumTicks ) + ';' );
end;

function TKOLTrackBar.SetupParams(const AName, AParent: String): String;
var S: String;
begin
  S := '';
  if trbAutoTicks      in Options then S := 'trbAutoTicks,';
  if trbEnableSelRange in Options then S := S + 'trbEnableSelRange,';
  if trbFixedLength    in Options then S := S + 'trbFixedLength,';
  if trbNoThumb        in Options then S := S + 'trbNoThumb,';
  if trbNoTicks        in Options then S := S + 'trbNoTicks,';
  if trbTooltips       in Options then S := S + 'trbTooltips,';
  if trbTopLeftMarks   in Options then S := S + 'trbTopLeftMarks,';
  if trbVertical       in Options then S := S + 'trbVertical,';
  if trbNoBorder       in Options then S := S + 'trbNoBorder,';
  S := Copy( S, 1, Length( S ) - 1 );
  Result := AParent + ', [' + S + '], ';
  if TMethod( OnScroll ).Code <> nil then
    Result := Result + 'Result.' + ParentForm.MethodName( TMethod( OnScroll ).Code )
  else
    Result := Result + 'nil';
end;

function TKOLTrackBar.TabStopByDefault: Boolean;
begin
  Result := TRUE;
end;

end.
